'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var parser = _interopDefault(require('yargs-parser'));
var menhera = require('menhera');
var chalk = _interopDefault(require('chalk'));
var events = require('events');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _toArray(arr) {
  return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var space = function space(num) {
  return Array.from(new Array(num)).join(" ");
};
var fillSpace = function fillSpace(size, current) {
  return space(size - current);
};
var genOutput = function genOutput(_ref) {
  var input = _ref.input,
      chalkFn = _ref.chalkFn,
      length = _ref.length,
      _ref$left = _ref.left,
      left = _ref$left === void 0 ? 0 : _ref$left,
      _ref$right = _ref.right,
      right = _ref$right === void 0 ? 0 : _ref$right;
  var inputLenght = input.length;
  var output = chalkFn(input);
  (!length || inputLenght > length) && (length = inputLenght);
  return "".concat(space(left)).concat(output || []).concat(fillSpace(length, inputLenght)).concat(space(right));
};
var ENV = function ENV(_ref2) {
  var $0 = _ref2.$0,
      _args = _ref2._args,
      args = _ref2.args,
      options = _ref2.options;
  var env = {};
  env.NONE_ARGS = args.length === 0;
  env.NONE_OPTIONS = Object.keys(options).length === 0;
  env.NONE_INPUTS = env.NONE_ARGS && env.NONE_OPTIONS;
  env.NONE_FULL_ARGS = _args.length !== args.length;
  return env;
};

var commands = {
  $: function $(_ref) {
    var _this = this;

    var _key = _ref._key,
        _val = _ref._val,
        cp = _ref.cp;
    var rootAlias = this.config.rootAlias;
    var name = _val.name,
        _val$args = _val.args,
        args = _val$args === void 0 ? [] : _val$args,
        _val$desc = _val.desc,
        cDesc = _val$desc === void 0 ? "" : _val$desc,
        _val$options = _val.options,
        options = _val$options === void 0 ? {} : _val$options;
    var key = name || _key;
    this.commands[key] = _val;
    var commandOutput = [];
    args = args.map(function (argv) {
      return "[".concat(argv, "]");
    });

    if (key !== rootAlias) {
      commandOutput.push("".concat(genOutput({
        input: "".concat(key),
        chalkFn: chalk.magenta,
        length: 8,
        left: 2
      })).concat(genOutput({
        input: "".concat(args.join(" ")),
        chalkFn: chalk.grey,
        length: 40
      })).concat(genOutput({
        input: "".concat(cDesc),
        chalkFn: chalk.grey,
        length: 10
      }), "\n"));
    }

    var optionOutput = [];

    menhera.$(options, function (option, val) {
      var alias = val.alias,
          oDesc = val.desc;
      _this.alias[option] = alias;
      _this.alias[alias] = option;
      optionOutput.push("".concat(genOutput({
        input: "-".concat(alias),
        chalkFn: chalk.grey,
        length: 5,
        left: 10
      })).concat(genOutput({
        input: "--".concat(option),
        chalkFn: chalk.grey,
        length: 35
      })).concat(genOutput({
        input: oDesc,
        chalkFn: chalk.grey,
        length: 10
      }), "\n"));
    });

    menhera.$set(this.helper, _defineProperty({}, key, {
      optionOutput: optionOutput,
      commandOutput: commandOutput
    }));
  }
};
var config = {
  _: function _(_ref2) {
    var _2 = _ref2._,
        _val = _ref2._val;
    menhera.$set(this.config, _val);
  },
  start: function start(_ref3) {
    var _this2 = this;

    var _ = _ref3._,
        _val = _ref3._val;
    var _this$config = this.config,
        target = _this$config.target,
        rootAlias = _this$config.rootAlias;

    var _parser = parser(target || process.argv.slice(2)),
        __ = _parser._,
        options = _objectWithoutProperties(_parser, ["_"]);

    var _3 = _toArray(__),
        _$2 = _3[0],
        _key = _$2 === void 0 ? rootAlias : _$2,
        args = _3.slice(1);

    menhera.$(options, function (key, val) {
      _this2.args[key] = val;
      var alias = _this2.alias[key];

      if (alias) {
        _this2.args[alias] = val;
      }
    });

    var command = this.commands[_key] || {};

    var _command$args = command.args,
        _args = _command$args === void 0 ? [] : _command$args;

    _args.forEach(function (argv, i) {
      _this2.args[argv] = args[i];
    });

    args.forEach(function (argv, i) {
      _this2.args["$".concat(i)] = args[i];
    });
    var out = Object.assign({
      _: _
    }, this, this.args, {
      _key: _key,
      options: options,
      args: args
    });
    var val = Object.assign({}, out, {
      env: ENV(Object.assign({}, out, {
        _args: _args
      }))
    });
    var _command$exec = command.exec,
        exec = _command$exec === void 0 ? function () {} : _command$exec;
    var help = val.help,
        v = val.v;

    if (help) {
      _.$use({
        CLI: {
          help: _key
        }
      });

      return;
    }

    if (v) {
      var version = this.config.version;
      console.log(version);
      return;
    }

    exec(val);
  }
};
function help(_ref4) {
  var _val = _ref4._val;
  var help;
  var rootAlias = this.config.rootAlias;

  if (_val !== rootAlias || !_val) {
    help = this.helper[_val];
  } else {
    help = {
      commandOutput: Object.values(this.helper).map(function (help) {
        return help.commandOutput;
      }),
      optionOutput: this.helper[rootAlias].optionOutput
    };
  }

  console.log("\n\n".concat(chalk.grey("Commands:"), "\n\n").concat(help.commandOutput.join(""), "\n\n").concat(chalk.grey("Options:"), "\n\n").concat(help.optionOutput.join(""), "\n    "));
}
function usage(_ref5) {
  var _val = _ref5._val;
  var help = this.helper[_val];
  help && console.log(" \n  \n".concat(chalk.grey("Usage:"), "\n              \n    ").concat(help.commandOutput.join("")));
}
var Message = {
  $: function $(_ref6) {
    var _key = _ref6._key,
        _val = _ref6._val;
    var messages = this.messages[_key] || {};
    var message = messages[_val] || "";

    if (message) {
      typeof message === "function" && console.log(message(this));
      typeof message !== "function" && console.log(message);
    } else {
      console.log("can not find message with key: ".concat(_key, " val: ").concat(_val));
    }
  }
};
var messages = {
  _: function _(_ref7) {
    var _val = _ref7._val;
    Object.assign(this.messages, _val);
  }
};

var CLI = /*#__PURE__*/Object.freeze({
  commands: commands,
  config: config,
  help: help,
  usage: usage,
  Message: Message,
  messages: messages
});

var index = {
  name: "CLI",
  Event: new events.EventEmitter(),
  alias: {
    help: "h",
    h: "help",
    version: "v",
    v: "version"
  },
  args: {},
  commands: {},
  messages: {},
  helper: {},
  _hooks: {
    CLI: CLI
  },
  config: {
    version: "0.0.1",
    rootAlias: "_"
  }
};

module.exports = index;
